/*
 * $Id: isc.h,v 0.4 1991/09/21 12:07:58 ceder Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
 * $Id: isc.h,v 0.4 1991/09/21 12:07:58 ceder Exp $
 *
** isc.h                        structures and defines used in a ISC server
**
** history:
** 910306 pen      major overhaul
** 910307 pen      type name changes, changes in structs..
*/

#ifndef __ISC_H__
#define __ISC_H__


#include <time.h>
#include <stdarg.h>


/*
** Give the poor user a chance to change it
*/
#ifndef ISC_UDGTYPE
#  define ISC_UDGTYPE void
#endif


/*
** Some alternative names to satisfy different tastes
*/
#define Isc_master      IscMaster     
#define ISCMCB		IscMaster     

#define Isc_session     IscSession  
#define ISCSCB	        IscSession  

#define Isc_event       IscEvent     
#define ISCECB		IscEvent      


/*
** Some nice defaults
*/
#define ISC_DEFAULT_MAX_MSG_SIZE      2048
#define ISC_DEFAULT_MAX_QUEUED_SIZE   300
#define ISC_DEFAULT_MAX_DEQUEUE_LEN   10


/*
** Not yet implemented
*/
typedef void IscConfig;


/*
** Various types of events that may happen
*/
typedef enum 
{
    ISC_EVENT_ERROR,
    ISC_EVENT_TIMEOUT,
    ISC_EVENT_LOGIN,
    ISC_EVENT_LOGOUT,
    ISC_EVENT_MESSAGE,
} IscEventType;


/*
** Various types of sessions
*/
typedef enum
{
  ISC_TYPE_UNKNOWN,
  ISC_TYPE_TCP,
  ISC_TYPE_PIPE,
  ISC_TYPE_FILE
} IscSessionType;


/*
** The Master Control Block
*/
typedef struct isc_mcb
{
  int listenc;
  struct isc_listen
  {
    IscSessionType    type;
    int               fd;
    void            * info;
  } *listenv;

  int maxmsgsize;
  int maxqueuedsize;
  int maxdequeuelen;
  
  struct isc_scb *sessions; /* Pointer into a circular list. */
} IscMaster;


/*
** Generic message type
*/
typedef struct isc_msg
{
  int size;      /* Maximum buffer size */
  int length;    /* Length of used buffer */
  char *buffer;  /* Pointer to buffer */
} IscMessage;


/*
** Entry in a message queue
*/
typedef struct isc_msg_q_entry
{
  struct isc_msg_q_entry  * prev;
  struct isc_msg_q_entry  * next;
  IscMessage              * msg;
} IscMsgQE;


/*
** The generic message queue
*/
typedef struct isc_msgqueue
{
  IscMsgQE  * head;
  IscMsgQE  * tail;
  int         entries;
} IscMsgQueue;



typedef struct isc_scb
{
    struct isc_scb * prev;
    struct isc_scb * next;

    IscMaster    * mcb;

    IscSessionType   type;
    int              fd;
    
    IscMsgQueue   * rd_msg_q;
    IscMsgQueue   * wr_msg_q;

    union
    {
      struct
      {
	int     listen;      /* Listen index, -1 if unused */
	char  * hostname;    /* Remote host */
	int     rport;	     /* Remote port */
	int     lport;       /* Local  port */
      } tcp;
      struct
      {
	char  * pathname;
	int     openmode;
      } file;
      struct
      {
	int     listen;      /* Listen index, -1 if unused */
	char  * pathname;
      } pipe;
    } info;

    
    /* Cheap transmit buffer */
    /* Should be dynamically allocated - into an IscMessage */
    char              sendbuf[2048];
    int               sendindex;

    int		      kill_him; /* Wrong protocol, too many tries with
				 * wrong password (IMPL) et c may cause
				 * the server to force the user to log
				 * off. */

    time_t            logintime;
    struct
    {
      struct
      {
	long bytes;
	long packets;
      } rx, tx;
    } stats;

    ISC_UDGTYPE     * udg;   /* User defined garbage :-) */
} IscSession;


typedef struct isc_ecb
{
  IscEventType    event;
  IscSession    * session;
  IscMessage    * msg;
} IscEvent;



/*
 * This routine will setup the ISC subsystem
 */
extern IscMaster *
isc_initialize(IscConfig  * cfg);



/*
** Shut down all sessions associated with an ISC Master Control Block
** and deallocate all storage used by the MCB.
*/
extern void
isc_shutdown(IscMaster  * mcb);



/*
** Establish a TCP port to listen for connections at
*/
extern int
isc_listentcp(IscMaster   * mcb,
	      const char  * service);



/*
** Stop to listen at a specified port
*/
extern int
isc_unlisten(IscMaster  * mcb,
	     int          index);



/*
** Initiate a new session with a previously opened file
**
** Note: (IscSession).type will be assigned ISC_TYPE_UNKNOWN.
*/
extern IscSession *
isc_openfd(IscMaster  * mcb,
	   int          fd);


/*
** Initiate a new session with a specified file
**
** Note: (IscSession).type will be assigned ISC_TYPE_FILE.
*/
extern IscSession *
isc_openfile(IscMaster   * mcb,
	     const char  * pathname,
	     int           openmode);


/*
** Initiate a new session with a remote TCP service
**
** Note: (IscSession).type will be assigned ISC_TYPE_FILE.
*/
extern IscSession *
isc_opentcp(IscMaster   * mcb,
	    const char  * hostname,
	    const char  * service);



/*
** When isc_getnextevent returns a ISC_EVENT_LOGOUT this function should
** be called. It can also be called if the server wants to log out a client.
*/
extern void
isc_close(IscSession  * scb);



/*
** This function returns the number of active sessions
*/
extern int
isc_sessions(IscMaster  * mcb);



/*
 * Wait for an event on a specific server. Timeout is specified in milliseconds
 */
extern IscEvent *
isc_getnextevent(IscMaster  * mcb,
		 long         timeout);


/*
 * This should be used to discard an event when it has been processed.
 */
extern void
isc_dispose(IscEvent  * ecb);



/*
** Flush the transmit queue for a specific session
*/
extern void
isc_flush(IscSession  * scb);


/*
** Add a new message to the transmit queue
*/
extern void
isc_send(IscSession  * scb, IscMessage  * mcb);


/*
** Put a buffer on the transmit queue for a session
*/
extern int
isc_write(IscSession  * scb,
	  const void  * buffer,
	  size_t        length);


/*
** Put a single character on the transmit queue
*/
extern int
isc_putc(int           chr,
	 IscSession  * scb);


/*
** Put a formatted string on the transmit queue
*/
extern int
isc_printf(IscSession  * scb,
	   const char  * format,
	   ...);

extern int
isc_vprintf(IscSession  * scb,
	    const char  * format,
	    va_list       AP);


extern IscMessage *
isc_allocmsg(size_t  size);

extern IscMessage *
isc_reallocmsg(IscMessage  * msg,
	       size_t        size);

extern void
isc_freemsg(IscMessage  * msg);

extern IscMessage *
isc_mkstrmsg(const char  * str);



/*
 * Set max size of messages for a server. If this function is not called a
 * default (currently 2048 bytes) is used.
 *
 * NOTE: This currently only applies to OUTGOING message, incoming
 *       is split into max 2048 byte each....
 */
extern void
isc_setmaxmsgsize(IscMaster  * mcb,
		  int       size);
    

extern void
isc_setlogfn(void (*logfn)(const char  * format, va_list AP));
			   


extern void
isc_setallocfn(void  * (*mallocfn)(size_t size),
               void  * (*reallocfn)(void  * buf, size_t size),
	       void    (*freefn)(void  * buf));

extern void
isc_setabortfn(void (*abortfn)(const char  * msg));

#endif /* __ISC_H__ */
