/*
 * $Id: isc_alloc.c,v 0.7 1991/10/16 17:16:47 ceder Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
** isc_alloc.c                               ISC storage allocation routines
**
** history:
** 910305 pen      moved into separate file
*/

static char *rcsid = "$Id: isc_alloc.c,v 0.7 1991/10/16 17:16:47 ceder Exp $";

#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "isc.h"
#include "isc-dump.h"
#include "intern.h"

/*
** Some magic numbers
*/
#define ISC_MAGIC_ALLOC	0x12F54ACE
#define ISC_MAGIC_FREE  0xEE47A37F

static long no_of_allocated_buffers = 0;

/*
** Pointers to functions to handle storage allocation
*/
static void *   (*isc_mallocfn)(size_t size)              = NULL;
static void *   (*isc_reallocfn)(void *bufp, size_t size) = NULL;
static void     (*isc_freefn)(void *bufp)                 = NULL;


/*
** Define functions to handle storage allocation
*/
void
isc_setallocfn(void * (*mallocfn)(size_t size),
             void * (*reallocfn)(void *buf, size_t size),
	     void   (*freefn)(void *buf))
{
  isc_mallocfn  = mallocfn;
  isc_reallocfn = reallocfn;
  isc_freefn    = freefn;
}



void *
isc_malloc(size_t size)
{
  unsigned int  * buf;


  if (isc_mallocfn)
    buf = (unsigned int *) (*isc_mallocfn)(size + sizeof(unsigned int));
  else
  {
    buf = (unsigned int *) malloc(size + sizeof(unsigned int));
    if (!buf)
    {
      isc_log("isc_malloc: failed allocating size = %d", size);
      isc_abort("isc_malloc");
    }
  }

  ++no_of_allocated_buffers;

  *buf++ = ISC_MAGIC_ALLOC;

  return (void *) buf; 
}


void *
isc_realloc(void *oldbuf, size_t size)
{
  unsigned int *buf;


  if (!oldbuf)
    return isc_malloc(size);

  buf = (unsigned int *) oldbuf;
  switch (*--buf)
  {
    case ISC_MAGIC_ALLOC:
      break;

    case ISC_MAGIC_FREE:
      isc_log("ISC_REALLOC: Trying to realloc freed block");
      isc_abort("isc_realloc");

    default:
      isc_log("ISC_REALLOC: Bad magic number");
      isc_abort("isc_realloc");
  }

  *buf = ISC_MAGIC_FREE;
  
  if (isc_reallocfn)
    buf = (unsigned int *) (*isc_reallocfn)((void *) buf,
					    size + sizeof(unsigned int));
  else
  {
    buf = (void *) realloc((void *) buf, size + sizeof(unsigned int));
    if (!buf)
    {
      isc_log("isc_realloc: failed allocating size = %d", size);
      isc_abort("isc_realloc");
    }
  }

  *buf++ = ISC_MAGIC_ALLOC;
  return (void *) buf;
}



void
isc_free(void *buf)
{
  unsigned int *ibuf;

  
  if (!buf)
  {
    isc_log("ISC_FREE: Trying to free NULL pointer");
    isc_abort("isc_free");
  }
    
  ibuf = (unsigned int *) buf;
  switch (*--ibuf)
  {
    case ISC_MAGIC_ALLOC:
      break;

    case ISC_MAGIC_FREE:
      isc_log("ISC_FREE: Trying to realloc freed block");
      isc_abort("isc_free");

    default:
      isc_log("ISC_FREE: Bad magic number");
      isc_abort("isc_free");
  }

  *ibuf = ISC_MAGIC_FREE;

  --no_of_allocated_buffers;

  if (isc_freefn)
  {
    (*isc_freefn)((void *) ibuf);
    return;
  }
  else
    free((void *) ibuf);
}



/*
** Safe (hopefully) string duplicator
*/
char *
isc_strdup(const char *str)
{
  char *newstr;

  
  newstr = isc_malloc(strlen(str)+1);
  strcpy(newstr, str);
  return newstr;
}

void
dump_isc_alloc_counts(FILE *stat_file)
{
    fprintf(stat_file, "---" __FILE__ ":\n"
	    "\tAllocated blocks by isc:    %d\n",
	    no_of_allocated_buffers);
}
