/*
 * $Id: isc_session.c,v 0.3 1991/09/15 10:23:48 linus Exp $
 * Copyright (C) 1991  Lysator Academic Computer Association.
 *
 * This file is part of the LysKOM server.
 * 
 * LysKOM is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 1, or (at your option) 
 * any later version.
 * 
 * LysKOM is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with LysKOM; see the file COPYING.  If not, write to
 * Lysator, c/o ISY, Linkoping University, S-581 83 Linkoping, SWEDEN,
 * or the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 * MA 02139, USA.
 *
 * Please mail bug reports to bug-lyskom@lysator.liu.se. 
 */
/*
** isc_session.c                              Routines to handle ISC sessions
**
** history:
** 910305 pen      moved into separate file
*/

static char *rcsid = "$Id: isc_session.c,v 0.3 1991/09/15 10:23:48 linus Exp $";

#include <stddef.h>
#include <stdlib.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <netdb.h>
#include <string.h>
#include <fcntl.h>

#include "isc.h"
#include "intern.h"





void
isc_addsession(IscMaster   * mcb,
	       IscSession  * scb)
{
    scb->mcb = mcb;

    /* Better check if this is the first session or not */
    if (mcb->sessions != NULL)
    {
        scb->next           = mcb->sessions;
	scb->prev           = mcb->sessions->prev;
	scb->prev->next     = scb;
	mcb->sessions->prev = scb;
    }
    else
    {
      scb->next    = scb;
      scb->prev    = scb;
    }
    mcb->sessions    = scb;
}



IscSession *
isc_newsession(int fd)
{
    IscSession  * scb;
    int           res;
    

    scb = (IscSession *) isc_malloc(sizeof(IscSession));

    scb->fd       = fd;
    scb->type     = ISC_TYPE_UNKNOWN;
    scb->prev     = NULL;
    scb->next     = NULL;
    scb->mcb      = NULL;
    scb->rd_msg_q = NULL;
    scb->wr_msg_q = NULL;
    scb->kill_him = 0;

    scb->stats.rx.bytes   = 0;
    scb->stats.rx.packets = 0;
    scb->stats.tx.bytes   = 0;
    scb->stats.tx.packets = 0;
    
  
    /* Set non blocking write mode */
    if ((res = fcntl(scb->fd, F_GETFL, 0)) == -1)
    {
        close(scb->fd);
        isc_free(scb);
	return NULL;
    }

    if (fcntl(scb->fd, F_SETFL, res | FNDELAY) == -1)
    {
        close(scb->fd);
	isc_free(scb);
	return NULL;
    }
	    
    /* Fill in the session structure */
    scb->rd_msg_q      = isc_newqueue();
    scb->wr_msg_q      = isc_newqueue();
    scb->sendindex     = 0;

    return scb;
}


void
isc_close(IscSession  * scb)
{
    if (!scb)
        return;
    
    if (scb->fd != -1)
    {
	close(scb->fd);
	scb->fd = -1;
    }

    /* Remove it from the list of sessions if it is still on it */
    if (scb->prev != NULL)
    {
	if (scb->next != scb)
	{
	    /* Make sure noone references this session */
	    if (scb->mcb->sessions == scb)
		scb->mcb->sessions = scb->next;

	    scb->prev->next = scb->next;
	    scb->next->prev = scb->prev;
	}
	else
	{
	    /* This was the last session on the circular list. */
	    scb->mcb->sessions = NULL;
	}

	scb->prev = NULL;
	scb->next = NULL;
    }
    else if (scb->next != NULL)
    {
	isc_log("isc_close: prev == NULL; yet next != NULL\n");
    }

    isc_killqueue(scb->rd_msg_q);
    isc_killqueue(scb->wr_msg_q);

    switch (scb->type)
    {
      case ISC_TYPE_UNKNOWN:
        break;
	
      case ISC_TYPE_TCP:
	isc_free(scb->info.tcp.hostname);
	break;
	
      case ISC_TYPE_FILE:
	isc_free(scb->info.file.pathname);
	break;
	
      case ISC_TYPE_PIPE:
	break;
    }
    isc_free(scb);
  
    return;
}


/*
** Return the number of active sessions
*/
int
isc_sessions(IscMaster  * mcb)
{
  IscSession  * scb;
  IscSession  * top_scb;
  int           cnt;
  

  top_scb = mcb->sessions;
  for (cnt = 0, scb = top_scb; scb; cnt++, scb = scb->next)
    if (scb->next == top_scb)
      break;

  return cnt;
}



IscSession *
isc_openfd(IscMaster  * mcb,
	   int          fd)
{
  IscSession *scb;

  
  scb = isc_newsession(fd);
  if (!scb)
      return NULL;

  time(&scb->logintime);

  isc_addsession(mcb, scb);
  return scb;
}


IscSession *
isc_openfile(IscMaster   * mcb,
	     const char  * pathname,
	     int           openmode)
{
  IscSession  * scb;
  int           fd;
  

  fd = open(pathname, openmode);
  if (fd == -1)
      return NULL;

  scb = isc_openfd(mcb, fd);
  if (!scb)
      return NULL;

  scb->type               = ISC_TYPE_FILE;
  scb->info.file.pathname = isc_strdup(pathname);
  scb->info.file.openmode = openmode;

  return scb;
}




/*
** Establish a session with a remote TCP service
*/
IscSession *
isc_opentcp(IscMaster   * mcb,
	    const char  * host,
	    const char  * service)
{
    IscSession          * scb;
    struct sockaddr_in    server;
    struct hostent      * hp;
    struct servent	* sp;
    int                   fd;


    /* Local connection? */
    if (host == NULL)
	host = "localhost";
    
    memset(&server, 0, sizeof(server));

    if ((hp = gethostbyname(host)) == NULL)
    {
	server.sin_family = AF_INET;
	server.sin_addr.s_addr = inet_addr(host);
	if (server.sin_addr.s_addr == -1)
	    return NULL;
    }
    else
    {
	server.sin_family = hp->h_addrtype;
	memcpy(&server.sin_addr, hp->h_addr, sizeof(server.sin_addr));
    }
      

    if ((sp = getservbyname(service, "tcp")) == NULL)
      server.sin_port = htons(atoi(service));
    else
      server.sin_port = sp->s_port;
    
    if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	return NULL;

    setsockopt(fd, SOL_SOCKET, SO_DONTLINGER, 0, 0); 
    setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, 0, 0);

    if (connect(fd, (struct sockaddr *) &server, sizeof(server)) < 0)
    {
	close(fd);
	return NULL;
    }


    scb = isc_openfd(mcb, fd);
    if (!scb)
      return NULL;

    scb->type = ISC_TYPE_TCP;
    scb->info.tcp.listen   = -1;
    scb->info.tcp.hostname = isc_strdup(host);
    scb->info.tcp.rport    = server.sin_port;
    scb->info.tcp.lport    = isc_getlportnum(fd); 
    
    return scb;
}




